<?php

namespace EventPortal\Tests\Unit\Util;

use Carbon\Carbon;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Response\Models\Response;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class MagicAccessorTest extends TestCase
{
    use RefreshDatabase;

    /** @test */
    function can_get_simple_value_from_model(): void
    {
        $event = factory(Event::class)->create();
        $section = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'Section',
            'label' => 'TestField',
        ]);
        $field = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'TextInput',
            'label' => 'Test Text',
            'name' => 'test',
            'section_id' => $section->id,
        ]);

        factory(Response::class)->create([
            'field_id' => $field->id,
            'data' => [
                'value' => '07700912345',
            ],
        ]);

        $this->assertEquals('07700912345', $event->test);
    }

    /** @test */
    function can_get_select_value_from_model(): void
    {
        Carbon::setTestNow(Carbon::parse('01-01-2020'));

        $event = factory(Event::class)->create();
        $section = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'Section',
            'label' => 'TestField',
        ]);

        $field = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'Select',
            'label' => 'Test Select',
            'name' => 'test',
            'section_id' => $section->id,
        ]);
        factory(Response::class)->create([
            'field_id' => $field->id,
            'data' => [
                'value' => [
                    'label' => 'Test Value',
                    'value' => 1,
                ],
            ],
        ]);

        $this->assertEquals('Test Value', $event->test);
    }

    /** @test */
    function can_get_multi_select_value_from_model(): void
    {
        Carbon::setTestNow(Carbon::parse('01-01-2020'));

        $event = factory(Event::class)->create();
        $section = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'Section',
            'label' => 'TestField',
        ]);

        $field = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'Select',
            'label' => 'Test Multi Select',
            'name' => 'test',
            'section_id' => $section->id,
        ]);
        
        factory(Response::class)->create([
            'field_id' => $field->id,
            'data' => [
                'value' => [
                    [
                        'label' => 'First Value',
                        'value' => 69,
                    ],
                    [
                        'label' => 'Second Value',
                        'value' => 420,
                    ],
                ],
            ],
        ]);

        $this->assertEquals('First Value, Second Value', $event->test);
    }

    /** @test */
    function will_use_model_property_first(): void
    {
        $expected = Carbon::parse('01-01-2020');

        Carbon::setTestNow($expected);
        $event = factory(Event::class)->create();

        Carbon::setTestNow(Carbon::parse('02-01-2020'));
        $section = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'Section',
            'label' => 'TestField',
        ]);

        Carbon::setTestNow(Carbon::parse('03-01-2020'));
        $field = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'TextData',
            'label' => 'Created At',
            'name' => 'created_at',
            'section_id' => $section->id,
        ]);

        factory(Response::class)->create([
            'field_id' => $field->id,
            'data' => [
                'value' => [
                    'label' => 'Created At',
                    'value' => now(),
                ],
            ],
        ]);

        Carbon::setTestNow(Carbon::parse('04-01-2020'));


        $this->assertEquals($expected, $event->created_at);
    }

    /** @test */
    function can_handle_empty_values_gracefully(): void
    {
        $event = factory(Event::class)->create();
        $section = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'Section',
            'label' => 'TestField',
        ]);
        factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'Select',
            'label' => 'Test Field',
            'name' => 'test',
            'section_id' => $section->id,
        ]);

        $this->assertNull($event->invalidVariableName);
        $this->assertNull($event->test);
    }
}
